#################################################
########## BOOTSTRAP REGRESSION #################
#################################################

#-- Set seed

set.seed(4)

#-- Parameters

Sample<-seq(0,1,by=1/15)[1:15]
Sample_size=length(Sample)

Beta_0<-1
Beta_1<-1

Bootstrap_resample_size<-500
Matrix_bootstrap_errors<-matrix(0,nrow=Sample_size,ncol=Bootstrap_resample_size)
Montecarlo_size<-500

Alpha<-0.05

IC_Montecarlo_bootstrap_student_errors<-matrix(0,nrow=Montecarlo_size,ncol=2)
IC_Montecarlo_student_student_errors<-matrix(0,nrow=Montecarlo_size,ncol=2)
IC_Montecarlo_bootstrap_normal_errors<-matrix(0,nrow=Montecarlo_size,ncol=2)
IC_Montecarlo_student_normal_errors<-matrix(0,nrow=Montecarlo_size,ncol=2)

#-- Head 1), 2) and 3)

Errors<-rt(Sample_size,3)
Response_values<-Beta_0+Beta_1*Sample+Errors

Hat_beta_1<-(sum((Sample-mean(Sample))*Response_values))/((Sample_size-1)*var(Sample))
Hat_beta_0<-mean(Response_values)-Hat_beta_1*mean(Sample)
Residuals<-Response_values-Hat_beta_0-Hat_beta_1*Sample

Residuals_modificated<-Residuals/sqrt(1-(1/Sample_size+((Sample-mean(Sample))^2)/((Sample_size-1)*var(Sample))))

Matrix_bootstrap_errors<-matrix(sample(Residuals_modificated-mean(Residuals_modificated),size=Sample_size*Bootstrap_resample_size,replace=T),nrow=Sample_size,ncol=Bootstrap_resample_size)

Response_values_bootstrap<-Hat_beta_0+Hat_beta_1*Sample+Matrix_bootstrap_errors
Hat_beta_1_bootstrap<-apply((Sample-mean(Sample))*Response_values_bootstrap,2,sum)/((Sample_size-1)*var(Sample))
Hat_beta_0_bootstrap<-apply(Response_values_bootstrap,2,mean)-Hat_beta_1_bootstrap*mean(Sample)
Residuals_bootstrap<-t(Response_values_bootstrap)-Hat_beta_0_bootstrap-outer(Hat_beta_1_bootstrap,Sample)

T_statistic_bootstrap<-(Hat_beta_1_bootstrap-Hat_beta_1)/sqrt((apply(t(Residuals_bootstrap)^2,2,sum)/(Sample_size-2))/((Sample_size-1)*var(Sample)))
	
IC_bootstrap<-c(Hat_beta_1-sort(T_statistic_bootstrap)[floor((1-Alpha/2)*Bootstrap_resample_size)]*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))),Hat_beta_1-sort(T_statistic_bootstrap)[floor(Alpha/2*Bootstrap_resample_size)]*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))))

((IC_bootstrap[1]<Beta_1)&(Beta_1<IC_bootstrap[2]))


#-- Head 4) and 5)

for(m in 1:Montecarlo_size){

Errors<-rt(Sample_size,3)
Response_values<-Beta_0+Beta_1*Sample+Errors

Hat_beta_1<-(sum((Sample-mean(Sample))*Response_values))/((Sample_size-1)*var(Sample))
Hat_beta_0<-mean(Response_values)-Hat_beta_1*mean(Sample)
Residuals<-Response_values-Hat_beta_0-Hat_beta_1*Sample

Residuals_modificated<-Residuals/sqrt(1-(1/Sample_size+((Sample-mean(Sample))^2)/((Sample_size-1)*var(Sample))))
Matrix_bootstrap_errors<-matrix(sample(Residuals_modificated-mean(Residuals_modificated),size=Sample_size*Bootstrap_resample_size,replace=T),nrow=Sample_size,ncol=Bootstrap_resample_size)

Response_values_bootstrap<-Hat_beta_0+Hat_beta_1*Sample+Matrix_bootstrap_errors
Hat_beta_1_bootstrap<-apply((Sample-mean(Sample))*Response_values_bootstrap,2,sum)/((Sample_size-1)*var(Sample))
Hat_beta_0_bootstrap<-apply(Response_values_bootstrap,2,mean)-Hat_beta_1_bootstrap*mean(Sample)
Residuals_bootstrap<-t(Response_values_bootstrap)-Hat_beta_0_bootstrap-outer(Hat_beta_1_bootstrap,Sample)

T_statistic_bootstrap<-(Hat_beta_1_bootstrap-Hat_beta_1)/sqrt(( apply(t(Residuals_bootstrap)^2,2,sum))/((Sample_size-2) * (Sample_size-1)*var(Sample)))

IC_Montecarlo_bootstrap_student_errors[m,]<-c(Hat_beta_1-(sort(T_statistic_bootstrap)[floor((1-Alpha/2)*Bootstrap_resample_size)])*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))),Hat_beta_1-sort(T_statistic_bootstrap)[floor(Alpha/2*Bootstrap_resample_size)]*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))))

IC_Montecarlo_student_student_errors[m,]<-c(Hat_beta_1-qt(1-Alpha/2,df=Sample_size-2)*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))),Hat_beta_1+qt(1-Alpha/2,df=Sample_size-2)*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))))
}

(Coverage_bootstrap_student_errors<-sum((IC_Montecarlo_bootstrap_student_errors[,1]<=1)&(IC_Montecarlo_bootstrap_student_errors[,2]>=1))/Montecarlo_size)
(Coverage_student_student_errors<-sum((IC_Montecarlo_student_student_errors[,1]<=1)&(IC_Montecarlo_student_student_errors[,2]>=1))/Montecarlo_size)


# Intervals plots

ind<-1:Montecarlo_size

windows()

plot(IC_Montecarlo_bootstrap_student_errors[,2]~ind,type="n",ylim=c(min(IC_Montecarlo_bootstrap_student_errors[,1]),max(IC_Montecarlo_bootstrap_student_errors[,2])),xlab="Montecarlo Size",ylab="Bootstrap Intervals")

for(i in 1:Montecarlo_size){
segments(ind[i],IC_Montecarlo_bootstrap_student_errors[i,1],ind[i],IC_Montecarlo_bootstrap_student_errors[i,2])
}

abline(h=1,col=2,lwd=2)

windows()
boxplot(IC_Montecarlo_bootstrap_student_errors[,2]-IC_Montecarlo_bootstrap_student_errors[,1],ylim=c(0,50))



windows()

plot(IC_Montecarlo_student_student_errors[,2]~ind,type="n",ylim=c(min(IC_Montecarlo_student_student_errors[,1]),max(IC_Montecarlo_student_student_errors[,2])),xlab="Montecarlo Size",ylab="Student Intervals")

for(i in 1:Montecarlo_size){
segments(ind[i],IC_Montecarlo_student_student_errors[i,1],ind[i],IC_Montecarlo_student_student_errors[i,2])
}

abline(h=1,col=2,lwd=2)

windows()
boxplot(IC_Montecarlo_student_student_errors[,2]-IC_Montecarlo_student_student_errors[,1],ylim=c(0,30))



#-- Head 6)

for(m in 1:Montecarlo_size){

Errors<-rnorm(Sample_size,mean=0,sd=sqrt(3))
Response_values<-Beta_0+Beta_1*Sample+Errors

Hat_beta_1<-(sum((Sample-mean(Sample))*Response_values))/((Sample_size-1)*var(Sample))
Hat_beta_0<-mean(Response_values)-Hat_beta_1*mean(Sample)
Residuals<-Response_values-Hat_beta_0-Hat_beta_1*Sample

Residuals_modificated<-Residuals/sqrt(1-(1/Sample_size+((Sample-mean(Sample))^2)/((Sample_size-1)*var(Sample))))
Matrix_bootstrap_errors<-matrix(sample(Residuals_modificated-mean(Residuals_modificated),size=Sample_size*Bootstrap_resample_size,replace=T),nrow=Sample_size,ncol=Bootstrap_resample_size)

Response_values_bootstrap<-Hat_beta_0+Hat_beta_1*Sample+Matrix_bootstrap_errors
Hat_beta_1_bootstrap<-apply((Sample-mean(Sample))*Response_values_bootstrap,2,sum)/((Sample_size-1)*var(Sample))
Hat_beta_0_bootstrap<-apply(Response_values_bootstrap,2,mean)-Hat_beta_1_bootstrap*mean(Sample)
Residuals_bootstrap<-t(Response_values_bootstrap)-Hat_beta_0_bootstrap-outer(Hat_beta_1_bootstrap,Sample)
	
T_statistic_bootstrap<-(Hat_beta_1_bootstrap-Hat_beta_1)/sqrt(( apply(t(Residuals_bootstrap)^2,2,sum))/((Sample_size-2) * (Sample_size-1)*var(Sample)))

IC_Montecarlo_bootstrap_normal_errors[m,]<-c(Hat_beta_1-(sort(T_statistic_bootstrap)[floor((1-Alpha/2)*Bootstrap_resample_size)])*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))),Hat_beta_1-sort(T_statistic_bootstrap)[floor(Alpha/2*Bootstrap_resample_size)]*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))))

IC_Montecarlo_student_normal_errors[m,]<-c(Hat_beta_1-qt(1-Alpha/2,df=Sample_size-2)*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))),Hat_beta_1+qt(1-Alpha/2,df=Sample_size-2)*sqrt((sum(Residuals^2)/(Sample_size-2))/((Sample_size-1)*var(Sample))))
}

(Coverage_bootstrap_normal_errors<-sum((IC_Montecarlo_bootstrap_normal_errors[,1]<=1)&(IC_Montecarlo_bootstrap_normal_errors[,2]>=1))/Montecarlo_size)
(Coverage_student_normal_errors<-sum((IC_Montecarlo_student_normal_errors[,1]<=1)&(IC_Montecarlo_student_normal_errors[,2]>=1))/Montecarlo_size)

# Intervals plots

windows()

ind<-1:Montecarlo_size

plot(IC_Montecarlo_bootstrap_normal_errors[,2]~ind,type="n",ylim=c(min(IC_Montecarlo_bootstrap_normal_errors[,1]),max(IC_Montecarlo_bootstrap_normal_errors[,2])),xlab="Montecarlo Size",ylab="Bootstrap Intervals")

for(i in 1:Montecarlo_size){
segments(ind[i],IC_Montecarlo_bootstrap_normal_errors[i,1],ind[i],IC_Montecarlo_bootstrap_normal_errors[i,2])
}
abline(h=1,col=2,lwd=2)

windows()

plot(IC_Montecarlo_student_normal_errors[,2]~ind,type="n",ylim=c(min(IC_Montecarlo_student_normal_errors[,1]),max(IC_Montecarlo_student_normal_errors[,2])),xlab="Montecarlo Size",ylab="Student Intervals")

for(i in 1:Montecarlo_size){
segments(ind[i],IC_Montecarlo_student_normal_errors[i,1],ind[i],IC_Montecarlo_student_normal_errors[i,2])
}
abline(h=1,col=2,lwd=2)


